@@ -3,11 +3,12 @@
  *                                                                    
  * Xnee enables recording and replaying of X protocol data           
  *                                                                   
- *        Copyright (C) 1999, 2000, 2001, 2002, 2003 Henrik Sandklef                    
+ * Copyright (C) 1999, 2000, 2001, 2002, 
+ *               2003, 2004, 2005, 2006, 2007 Henrik Sandklef 
  *                                                                   
  * This program is free software; you can redistribute it and/or     
  * modify it under the terms of the GNU General Public License       
- * as published by the Free Software Foundation; either version 2    
+ * as published by the Free Software Foundation; either version 3    
  * of the License, or any later version.                             
  *                                                                   
  *                                                                   
@@ -18,8 +19,8 @@
  *                                                                   
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software       
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston,            
- * MA  02111-1307, USA.                                              
+ * Foundation, Inc., 51 Franklin Street, Boston,            
+ * MA  02110-1301, USA.                                              
  ****/
 
 
@@ -29,13 +30,42 @@
 #include "libxnee/xnee_record.h"
 #include "libxnee/xnee_replay.h"
 #include "libxnee/xnee_km.h"
+#include "libxnee/feedback.h"
 #include "libxnee/xnee_setget.h"
+#include "libxnee/xnee_grab.h"
+#include "libxnee/xnee_grab.h"
+#include "libxnee/feedback.h"
 
 
-/* internal prototypes */
-int
-get_modifier_sub(xnee_data *xd,  char *mod_str);
+int xkm_rem_blanks (char *array);
 
+const char *
+xnee_symbolic_modifier2modifier (xnee_data *xd, const char *mod_str)
+{
+   xnee_verbose((xd, "--- xnee_symbolic_modifier2modifier\n"));
+   if (xnee_check(mod_str, XNEE_SYMBOLIC_ALT, XNEE_SYMBOLIC_ALT))
+    {
+      return XNEE_REAL_ALT;
+    }
+  else if (xnee_check(mod_str, XNEE_SYMBOLIC_SHIFT, XNEE_SYMBOLIC_SHIFT))
+    {
+      return XNEE_REAL_SHIFT;
+    }
+  else if (xnee_check(mod_str, XNEE_SYMBOLIC_CONTROL, XNEE_SYMBOLIC_CONTROL))
+    {
+      return XNEE_REAL_CONTROL;
+    }
+  else if (xnee_check(mod_str, XNEE_SYMBOLIC_CAPS, XNEE_SYMBOLIC_CAPS))
+    {
+      return XNEE_REAL_CAPS;
+    }
+  else if (xnee_check(mod_str, XNEE_SYMBOLIC_SCROLL, XNEE_SYMBOLIC_SCROLL))
+    {
+      return XNEE_REAL_SCROLL;
+    }
+  else 
+    return NULL;
+}
 
 int
 xkm_rem_blanks (char *array)
@@ -61,10 +91,6 @@ xkm_rem_blanks (char *array)
   return XNEE_OK;
 }
 
-
-
-
-
 int
 get_modifier_from_mapping_sub(Display *display, char *mod_str) 
 {
@@ -73,8 +99,11 @@ get_modifier_from_mapping_sub(Display *display, char *mod_str)
   KeySym my_keysym;
   int i, j, max_keys;
   int ret=0;
-  static char *mod_list[]={"ShiftMask", "LockMask","ControlMask", 
-		  "Mod1Mask", "Mod2Mask", "Mod3Mask", "Mod4Mask", "Mod5Mask" };
+
+  static char *mod_list[]={
+    "ShiftMask", "LockMask","ControlMask", 
+    "Mod1Mask",  "Mod2Mask", "Mod3Mask", 
+    "Mod4Mask", "Mod5Mask" };
 
   mod_keymap = XGetModifierMapping(display);
   max_keys = mod_keymap->max_keypermod;
@@ -85,9 +114,11 @@ get_modifier_from_mapping_sub(Display *display, char *mod_str)
 
       if (XKeysymToString(my_keysym)!=NULL)
 	{
-	  if (xnee_check (mod_str, XKeysymToString(my_keysym),XKeysymToString(my_keysym)))
+	  if (xnee_check (mod_str, 
+			  XKeysymToString(my_keysym),
+			  XKeysymToString(my_keysym)))
 	    {
-	      ret |= get_modifier_sub(NULL, mod_list[i]);
+	      ret |= get_modifier_sub(NULL, mod_list[i], NULL);
 	      return ret;
 	    }
 	  else
@@ -147,7 +178,8 @@ get_modifier_from_mapping(Display *display, char *mod_str)
 }
 
 
-int
+
+xnee_keymask
 get_modifier_sub(xnee_data *xd,  char *mod_str) 
 {
   int ret=0;
@@ -208,8 +240,8 @@ get_modifier_sub(xnee_data *xd,  char *mod_str)
 }
 
 
-
-int 
+#ifdef USE_OBSOLETE
+static int 
 is_last (xnee_data *xd,  char *mod_strs) 
 {
   char *tmp;
@@ -217,7 +249,8 @@ is_last (xnee_data *xd,  char *mod_strs)
   int len = strlen (mod_strs);
   tmp=mod_strs;
 
-  xnee_verbose ((xd, " ---> is_last\n"));
+  xnee_verbose ((xd, " ---> is_last ('%s' %d)\n", mod_strs, len));
+
   for (i=0;  (i<len) ; i++)
     {
       if (tmp[i]=='+')
@@ -229,14 +262,15 @@ is_last (xnee_data *xd,  char *mod_strs)
   return 1;
 }
 
-int
-get_modifier(xnee_data *xd,  char *mod_strs) 
+xnee_keymask
+get_modifier(xnee_data *xd,  char *mod_strs, xnee_km_tuple *km) 
 {
+  #define MODIFIER_BUF_SIZE 50
   int ret=0;
   /*  int try_ret;*/
   char *tmp;
-  char mod_head[20]="";
-  char mod_tail[40]="";
+  char mod_head[MODIFIER_BUF_SIZE]="";
+  char mod_tail[MODIFIER_BUF_SIZE]="";
   int i;
   /*  int last=0;*/
   int len=strlen(mod_strs);
@@ -246,7 +280,8 @@ get_modifier(xnee_data *xd,  char *mod_strs)
   xkm_rem_blanks(mod_strs);
 
   _level++;
-  
+
+
   xnee_verbose((xd, "%d ---> get_modifier:\"%s\"\n", level, mod_strs));
   tmp=mod_strs;
   
@@ -259,14 +294,18 @@ get_modifier(xnee_data *xd,  char *mod_strs)
 	  if (tmp[i]=='+')
 	    {
 	      
-	      xnee_verbose((xd, "%d --- get_modifier 3:\n", level));
+	      xnee_verbose((xd, "%d --- get_modifier 3: '%s'\n", level, tmp));
 	      found=1;
 	      strncpy (mod_head, tmp, i); 
 	      tmp+=(sizeof(char)*i+1);
-	      
-	      strcpy (mod_tail, tmp); 
-	      ret |= get_modifier_sub (xd, mod_head);
-	      ret |= get_modifier (xd, mod_tail);
+
+	      xnee_verbose((xd, "%d --- get_modifier 4: '%s'\n", level, mod_head));
+	      strncpy (mod_tail, MODIFIER_BUF_SIZE - strlen(mod_tail), tmp); 
+
+	      xnee_verbose((xd, "%d --- get_modifier 5: '%s'\n", level, mod_tail));
+	      ret |= get_modifier_sub (xd, mod_head, km);
+
+	      ret |= get_modifier (xd, mod_tail, km);
 	      xnee_verbose((xd, "%d ---- get_modifier FOUND \"%s\"   ret=%d\n", level, tmp, ret));
 	      break;
 	    }
@@ -274,7 +313,7 @@ get_modifier(xnee_data *xd,  char *mod_strs)
     }
   else
     {
-      ret |= get_modifier_sub (xd, tmp);
+      ret |= get_modifier_sub (xd, tmp, km);
       xnee_verbose((xd, "%d ---- get_modifier LAST :\"%s\"  ret=%d\n", level, tmp, ret));
       
     }
@@ -286,6 +325,7 @@ get_modifier(xnee_data *xd,  char *mod_strs)
   return ret;
 }
 
+#endif
 
 
 /**************************************************************
@@ -294,7 +334,11 @@ get_modifier(xnee_data *xd,  char *mod_strs)
  *                                                            *
  *                                                            *
  **************************************************************/
-int
+/* 
+ *  OBSOLETE 
+ *
+ *
+xnee_keymask
 xget_modifier(xnee_data *xd, char *mod_str) 
 {
   int ret=-1;
@@ -347,10 +391,10 @@ xget_modifier(xnee_data *xd, char *mod_str)
   return ret;
 }
 
-
+*/
 
 int
-xnee_check_km(xnee_data *xd)
+xnee_check_key(xnee_data *xd)
 {
   XEvent ev;
 
@@ -358,7 +402,6 @@ xnee_check_km(xnee_data *xd)
        &&
        ( xd->grab_keys->grab))
     {
-	  
       /*
        * Has the user pressed STOP (mod + key) 
        */
@@ -367,67 +410,51 @@ xnee_check_km(xnee_data *xd)
 		    SyncKeyboard,
 		    CurrentTime);
       XFlush (xd->grab);
-      
 
       if ( XCheckMaskEvent ( xd->grab, 0xffffffff , &ev) == True)
 	{
 	  XEvent my_event ;
 	  int    tmp_code;
-	  int    tmp_modifier;
 	  int    mode;
+	  int i ;
+
 
 	  XNextEvent (xd->grab, &my_event);
+
 	  if (my_event.xkey.send_event==1) 
 	    { 
 	      xnee_verbose ((xd , "send_event==true\n")); 
 	    }
 
 	  if (my_event.type == KeyPress )
-	    xnee_verbose ((xd , "##### KeyPress\n")); 
+	    {
+	      xnee_verbose ((xd , "##### KeyPress\n")); 
+	    }
 	  else if (my_event.type == KeyRelease )
-	    xnee_verbose ((xd , "#### KeyRelease\n")); 
+	    {
+	      xnee_verbose ((xd , "#### KeyRelease\n")); 
+	    }
+
 
-	  tmp_modifier=my_event.xkey.state;
 	  tmp_code=my_event.xkey.keycode;
-	  xnee_verbose ((xd, "key      = %d\n", tmp_modifier));
-	  xnee_verbose ((xd, "modifier = %d\n", tmp_code));
-	  mode=xnee_get_grab_mode (xd, tmp_code, tmp_modifier);
+
+	  xnee_verbose ((xd, "key = %d\n", tmp_code));
+	  mode = xnee_get_grab_mode (xd, tmp_code);
+
 	  xnee_verbose ((xd, "mode     = %d\n", mode));
 	  
-	  
-	  xnee_verbose((xd, "\n\n\tUSER PUSHED MOD + KEY ... leaving\n\n\n"));
-	  if ( ( tmp_modifier == xd->grab_keys->stop_mod ) &&
-	       ( tmp_code == xd->grab_keys->stop_key ) )
+	  for (i=XNEE_GRAB_STOP;i<XNEE_GRAB_LAST;i++)
 	    {
-	      xd->grab_keys->grabbed_action=XNEE_GRAB_STOP;
+	      xnee_verbose((xd, "     compare %d == %d",tmp_code, xd->grab_keys->action_keys[i].key));
+	      if ( tmp_code == xd->grab_keys->action_keys[i].key )
+		{
+		  xd->grab_keys->grabbed_action = i ; 
+		  xnee_verbose((xd, "\n\n\tUSER PUSHED KEY ... i=%d\n\n\n",i));
+		  return XNEE_GRAB_DATA ; 
+		}	    
 	    }
-	  else if ( ( tmp_modifier == xd->grab_keys->pause_mod ) &&
-	       ( tmp_code == xd->grab_keys->pause_key ) )
-	    {
-	      xd->grab_keys->grabbed_action=XNEE_GRAB_PAUSE;
-	    }
-	  else if ( ( tmp_modifier == xd->grab_keys->resume_mod ) &&
-	       ( tmp_code == xd->grab_keys->resume_key ) )
-	    {
-	      xd->grab_keys->grabbed_action=XNEE_GRAB_RESUME;
-	    }
-	  else if ( ( tmp_modifier == xd->grab_keys->insert_mod ) &&
-	       ( tmp_code == xd->grab_keys->insert_key ) )
-	    {
-	      xd->grab_keys->grabbed_action=XNEE_GRAB_INSERT;
-	    }
-	  else 
-	    {
-	      xd->grab_keys->grabbed_action=XNEE_GRAB_NODATA;
-	    }
-	  return XNEE_GRAB_DATA;
-	}
-      else
-	{
-
 	  return XNEE_GRAB_NODATA;
 	}
-      
     }
   else
     {
@@ -438,38 +465,101 @@ xnee_check_km(xnee_data *xd)
 
 
 int 
-xnee_handle_km(xnee_data *xd)
+xnee_handle_rec_key(xnee_data *xd)
 {
-  int ret=XNEE_OK;
-  xnee_verbose ((xd, " ---> xnee_handle_km\n"));
+
+  int    ret=XNEE_OK;
+  static int exec_ctr = 0; /* Keeps track of nr of exec grabs */
+  char *exec_prog ;
+  int len;
+
+  xnee_verbose ((xd, " ---> xnee_handle_rec_km\n"));
+
+  switch ( xd->grab_keys->grabbed_action )
+    {
+    case XNEE_GRAB_STOP :
+       feedback (xd, "Xnee stop received");
+      break;
+
+    case XNEE_GRAB_PAUSE :
+      feedback (xd, "Xnee pause received");
+      break;
+
+    case XNEE_GRAB_RESUME :
+      feedback (xd, "Xnee resume received");
+      break;
+
+    case XNEE_GRAB_INSERT :
+      feedback (xd, "Xnee insert received");
+      break;
+
+    case XNEE_GRAB_EXEC :
+      exec_ctr++;
+      feedback (xd, "Xnee exec received");
+      break;
+
+    default:
+      ret=XNEE_NO_GRAB_DATA;
+      break;
+      
+    }
+
 
   if (xd->grab_keys->grabbed_action==XNEE_GRAB_STOP)
     {
-      xnee_verbose ((xd, " ---  xnee_handle_km: STOP \n"));
-      ret=XNEE_GRAB_STOP;
-      xnee_verbose ((xd, " <--- xnee_handle_km\n"));
+      xnee_verbose ((xd, " ---  xnee_handle_rec_km: STOP \n"));
+      xnee_verbose ((xd, " <--- xnee_handle_rec_km\n"));
     }
   else if (xd->grab_keys->grabbed_action==XNEE_GRAB_PAUSE)
     {
-      xnee_verbose ((xd, " ---  xnee_handle_km: PAUSE \n"));
+      xnee_verbose ((xd, " ---  xnee_handle_rec_km: PAUSE \n"));
+      feedback (xd, "Xnee pause received");
       xnee_unsetup_recording (xd);
       ret=XNEE_GRAB_PAUSE;
     }
   else if (xd->grab_keys->grabbed_action==XNEE_GRAB_RESUME)
     {
-      xnee_verbose ((xd, " ---  xnee_handle_km: RESUME \n"));
+      xnee_verbose ((xd, " ---  xnee_handle_rec_km: RESUME \n"));
+      feedback (xd, "Xnee resume received");
       xnee_setup_recording (xd);
       ret=XNEE_GRAB_RESUME;
     }
+  else if (xd->grab_keys->grabbed_action==XNEE_GRAB_EXEC)
+    {
+      xnee_verbose ((xd, " ---  xnee_handle_rec_km: EXEC (nr %d)\n", exec_ctr));
+      len = strlen (xnee_get_exec_prog(xd));
+      len = len + 10 ; 
+    
+      exec_prog = (char*) calloc(len, sizeof(char));
+      
+      sprintf(exec_prog, "%s %d &", xnee_get_exec_prog(xd), exec_ctr);
+      
+      xnee_verbose ((xd, " ---  xnee_handle_rec_km: EXEC \n"));
+      feedback (xd, "Xnee exec received");
+
+      if (exec_prog==NULL)
+	{
+	  fprintf (xd->out_file, "%s   \n",XNEE_EXEC_MARK);
+	}
+      else
+	{
+	  fprintf (xd->out_file, "%s    %s\n", XNEE_EXEC_MARK, exec_prog);
+	  system ( exec_prog );
+	  free(exec_prog);
+	}
+
+      ret=XNEE_GRAB_EXEC;
+    }
   else if (xd->grab_keys->grabbed_action==XNEE_GRAB_INSERT)
     {
       time_t rawtime;
       struct tm * timeinfo;
 
-      xnee_verbose ((xd, " ---  xnee_handle_km: MARK \n"));
+      feedback (xd, "Xnee inserting mark in log file");
+      xnee_verbose ((xd, " ---  xnee_handle_rec_km: MARK \n"));
       time ( &rawtime );
       timeinfo = localtime ( &rawtime );
-      fprintf (xd->out_file, "#XNEE INSERT MARK:%.4d-%.2d-%.2d:%.2d.%.2d.%.2d\n",
+      fprintf (xd->out_file, "M MARK:%.4d-%.2d-%.2d:%.2d.%.2d.%.2d\n",
 	       timeinfo->tm_year + 1900 , 
 	       timeinfo->tm_mon + 1 , 
 	       timeinfo->tm_mday  ,
@@ -480,112 +570,169 @@ xnee_handle_km(xnee_data *xd)
     }
   else 
     {
-      xnee_verbose ((xd, " ---  xnee_handle_km: UNKNOWN \n"));
+      xnee_verbose ((xd, " ---  xnee_handle_rec_km: UNKNOWN \n"));
     }  
 
-  xnee_verbose ((xd, " <--- xnee_handle_km\n"));
+  xnee_verbose ((xd, " <--- xnee_handle_rec_km\n"));
+  return ret;
+}
+
+
+int 
+xnee_handle_rep_key(xnee_data *xd)
+{
+  int ret=XNEE_OK;
+  xnee_verbose ((xd, " ---> xnee_handle_rep_km\n"));
+
+  if (xd->grab_keys->grabbed_action==XNEE_GRAB_STOP)
+    {
+      xnee_verbose ((xd, " ---  xnee_handle_rep_km: STOP \n"));
+      xnee_verbose ((xd, " <--- xnee_handle_rep_km\n"));
+      feedback (xd, "Xnee stop received");
+      ret=XNEE_GRAB_STOP;
+    }
+  else if (xd->grab_keys->grabbed_action==XNEE_GRAB_PAUSE)
+    {
+      xnee_verbose ((xd, " ---  xnee_handle_rep_km: PAUSE \n"));
+      feedback (xd, "Xnee pause received");
+      fprintf (stderr, "pause during replay is not implemented yet\n");
+      ret=XNEE_GRAB_PAUSE;
+    }
+  else if (xd->grab_keys->grabbed_action==XNEE_GRAB_RESUME)
+    {
+      xnee_verbose ((xd, " ---  xnee_handle_rep_km: RESUME \n"));
+      feedback (xd, "Xnee resume received");
+      fprintf (stderr, "resume during replay is not implemented yet\n");
+      ret=XNEE_GRAB_RESUME;
+    }
+  else if (xd->grab_keys->grabbed_action==XNEE_GRAB_EXEC)
+    {
+      xnee_verbose ((xd, " ---  xnee_handle_rep_km: EXEC \n"));
+      feedback (xd, "Xnee exec received");
+      system ("xterm&");
+      ret=XNEE_GRAB_EXEC;
+    }
+  else if (xd->grab_keys->grabbed_action==XNEE_GRAB_INSERT)
+    {
+      feedback (xd, "Xnee inserting mark in log file");
+      xnee_verbose ((xd, " ---  xnee_handle_rep_km: MARK \n"));
+      fprintf (stderr, "insert during replay is not implemented yet\n");
+      ret=XNEE_GRAB_INSERT;
+    }
+  else 
+    {
+      xnee_verbose ((xd, " ---  xnee_handle_rep_km: UNKNOWN \n"));
+    }  
+
+  xnee_verbose ((xd, " <--- xnee_handle_rep_km\n"));
   return ret;
 }
 
 
 /**************************************************************
  *                                                            *
- * xnee_get_km_tuple                                          *
+ * xnee_get_key                                               *
  *                                                            *
  *                                                            *
  **************************************************************/
 int
-xnee_get_km_tuple (xnee_data     *xd, 
-		   xnee_km_tuple *km, 
-		   char          *mod_and_key)
+xnee_get_action_key (xnee_data     *xd, 
+		     xnee_action_key *ak, 
+		     char          *key)
 {
-  char key_buf[20];
-  char mod_buf[20];
-  int i=0;
-  int idx=-1;
-  int len=strlen(mod_and_key);
-  xnee_verbose((xd, " ---> xnee_get_km_tuple_key %s \n", mod_and_key));
-  strcpy (mod_buf, mod_and_key);
-  rem_all_blanks (mod_buf, strlen(mod_and_key));
+  KeySym ks;
 
-  /* make sure we have a display to grab on */
-  if ( xd->grab == NULL)
+  if (key==NULL)
     {
-      xd->grab = XOpenDisplay (xd->display);
-      if ( xd->grab == NULL)
-	{
-	  xnee_print_error ("Could not open display for grabbing\n");
-	  return (XNEE_NOT_OPEN_DISPLAY);
-	}
-    }
-  
-
-  for (i=0;i<len;i++)
-    {
-      if (mod_and_key[i] == '\0')
-	;
-      else if (mod_and_key[i] == ',')
-	idx=0;
-      if (idx == 0 )
-	{
-	  mod_buf[i]='\0';
-	  idx++;
-	}
-      else if (idx < 0 )
-	{
-	  mod_buf[i]=mod_and_key[i];
-	}
-      else
-	{
-	  key_buf[idx++ - 1]=mod_and_key[i];
-	  key_buf[idx-1]='\0';
-	}
+      ak->key=0;
     }
 
-  xnee_verbose((xd, " ---  xnee_get_km_tuple_key mod=\"%s\"   key=\"%s\"\n", mod_buf, key_buf));
-  /* Is it a modifier in alias form (e.g Control, Shift....)
-     or is it a number */
-  i=get_modifier(xd, mod_buf);
-  if ( i != -1 )
-    {
-      km->modifier=i;
-    }
-  else if ( sscanf ( mod_buf, "%d", &km->modifier) < 0 )
+  xnee_verbose((xd, " ---> xnee_get_action_key %s \n", key));
+
+  ks = XStringToKeysym (key)  ; 
+
+  if ( ks == 0 )
     {
       xnee_print_error ("SYNTAX ERROR\n\tCouldn't convert "
-			" string \"%s\" to an int\n",mod_buf);
+			"string \"%s\" to an int\n",key);
       return ( XNEE_SYNTAX_ERROR);
     }
+  xnee_verbose((xd, "  --- xnee_get_action_key %s gives us %d \n", 
+		key, (int)ks));
 
-  /* Is it a Key in letter form (e.g a,b)
-     or is it a number */
-  if ( sscanf ( key_buf, "%d", &km->key) <= 0 )
-    {
-      if (strlen(key_buf)!=1)
-	{
-	  xnee_print_error ("SYNTAX ERROR\n\tCouldn't convert "
-			    "string \"%s\" to an int.\n",key_buf);
-	  return ( XNEE_SYNTAX_ERROR);
-	}
-      else
-	{
-	  /* is it a letter */
-	  km->key = XStringToKeysym (key_buf)  ; 
-	  if ( km->key == 0 )
-	    {
-	      xnee_print_error ("SYNTAX ERROR\n\tCouldn't convert "
-				"string \"%s\" to an int\n",key_buf);
-	      return ( XNEE_SYNTAX_ERROR);
-	    }
-	  /* OK, it's a integer... just use it! */
-	  km->key = XKeysymToKeycode (xd->grab, km->key);
-	}
-    } 
-  xnee_verbose((xd, " <--- xnee_get_km_tuple_key %s \n", mod_and_key));
+  /* OK, it's a integer... just use it! */
+  ak->key = XKeysymToKeycode (xd->grab, ks);
+  
+  xnee_verbose((xd, " <--- xnee_get_action_key %s gives us %d \n", key, ak->key));
+  
   return XNEE_OK;
 }
 
 
+static int
+xnee_km_check_not_same(xnee_data *xd, char *str, int start)
+{
+  int ret  = 0 ;
+  char *tmp_ptr = NULL ; 
+
+
+  if (start!=XNEE_GRAB_UNKOWN)
+    {
+      if (start == XNEE_GRAB_STOP)
+	{
+	  return  XNEE_OK;
+	}
+      else if ( (start>0) && (start<XNEE_GRAB_LAST))
+	{
+	  tmp_ptr = xd->grab_keys->action_keys[start].str;
+	}
+      else
+	{
+	  return XNEE_OK;
+	}
+
+      if (str==NULL)
+	{
+	  return  XNEE_OK + xnee_km_check_not_same( xd, str, start + 1); 
+	}
+
+      if (tmp_ptr!=NULL)
+	{
+	  if (strcmp ( str, tmp_ptr)==0)
+	    {
+	      ret = 1;
+	    }
+	}
+      xnee_verbose((xd, "---  xnee_km_check_not_same %s %s %d ret =  %d\n",
+		str, tmp_ptr, start, ret ));
+
+      return ret + xnee_km_check_not_same( xd, str, start + 1); 
+    }
+  else 
+    {
+      xnee_verbose((xd, "---  xnee_km_check_not_same %s %s %d ret =  zero\n",
+		str, tmp_ptr, start ));
+
+      return 0 ;
+    }
+
+}
+
+int
+xnee_key_check(xnee_data *xd)
+{
+  int ret ; 
+  xnee_verbose((xd, " ---> xnee_km_check\n"));
+
+  ret = xnee_km_check_not_same(xd, 
+			       xd->grab_keys->action_keys[XNEE_GRAB_STOP].str, 
+			       XNEE_GRAB_STOP);
+  
+  xnee_verbose((xd, "  --- xnee_km_check %d \n", ret));
+
+  xnee_verbose((xd, " <--- xnee_km_check\n"));
+  return ret;
+}
 
 
 
