@@ -3,11 +3,11 @@
  *                                                                   
  * Xnee enables recording and replaying of X protocol data           
  *                                                                   
- *        Copyright (C) 1999, 2000, 2001, 2002, 2003 Henrik Sandklef                    
+ *    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004 Henrik Sandklef
  *                                                                   
  * This program is free software; you can redistribute it and/or     
  * modify it under the terms of the GNU General Public License       
- * as published by the Free Software Foundation; either version 2    
+ * as published by the Free Software Foundation; either version 3    
  * of the License, or any later version.                             
  *                                                                   
  *                                                                   
@@ -18,8 +18,8 @@
  *                                                                   
  * You should have received a copy of the GNU General Public License 
  * along with this program; if not, write to the Free Software       
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston,            
- * MA  02111-1307, USA.                                              
+ * Foundation, Inc., 51 Franklin Street, Boston,            
+ * MA  02110-1301, USA.                                              
  ****/
 
 /* This file contains function handling different resolution 
@@ -37,59 +37,35 @@ xnee_set_default_rep_resolution (xnee_data *xd)
 {
   if (xd->data != NULL)
     {
-      xd->res_info.replay.x_res = DisplayWidth (xd->data, 0);
+      xd->res_info.replay.x_res = DisplayWidth  (xd->data, 0);
       xd->res_info.replay.y_res = DisplayHeight (xd->data, 0);
       return XNEE_OK;
     }
   return -1 ;
 }
  
+int
+xnee_set_default_rec_resolution (xnee_data *xd)
+{
+  if ( (xd!=NULL) &&
+       (xd->data != NULL) )
+    {
+      xd->res_info.record.x_res = DisplayWidth  (xd->data, 0);
+      xd->res_info.record.y_res = DisplayHeight (xd->data, 0);
+      return XNEE_OK;
+    }
+  return -1 ;
+}
+ 
 
 int
 xnee_no_rep_resolution (xnee_data *xd)
 {
-  return ((xd->res_info.replay.x_res == 0) && 
-	  ( xd->res_info.replay.y_res == 0));
+  return ( (xd->res_info.replay.x_res != 1) && 
+	   ( xd->res_info.replay.y_res != 1));
 }
  
  
-int
-xnee_set_rec_resolution (xnee_data *xd, char *res_str)
-{
-  return xnee_str_to_res (res_str, &xd->res_info.record);
-}
- 
-int
-xnee_get_rec_resolution_x (xnee_data *xd)
-{
-  return xd->res_info.record.x_res;
-}
-
-int
-xnee_get_rec_resolution_y (xnee_data *xd)
-{
-  return xd->res_info.record.y_res;
-}
-
-
-int
-xnee_set_rep_resolution (xnee_data *xd, char *res_str)
-{
-  return xnee_str_to_res (res_str, &xd->res_info.replay);
-} 
-
-int
-xnee_get_rep_resolution_x (xnee_data *xd)
-{
-  return xd->res_info.replay.x_res;
-}
-
-int
-xnee_get_rep_resolution_y (xnee_data *xd)
-{
-  return xd->res_info.replay.y_res;
-}
- 
 
 int 
 xnee_str_to_res(char *res_str, xnee_res *xr)
@@ -97,8 +73,15 @@ xnee_str_to_res(char *res_str, xnee_res *xr)
   int ret;
 
   if (res_str == NULL)
-    return XNEE_BAD_RESOLUTION;
+    {
+      return XNEE_BAD_RESOLUTION;
+    }
 
+  if ( ! (  ( xr->x_res == 1 ) && ( xr->y_res == 1 )))
+    {
+      return XNEE_OK;
+    }
+  
   ret = sscanf(res_str, "%dx%d",
 	       &xr->x_res, 
 	       &xr->y_res); 
@@ -116,7 +99,7 @@ xnee_str_to_res(char *res_str, xnee_res *xr)
   else if (xnee_check ("SVGA", res_str, res_str))
     {
       xr->x_res = 1024 ;
-      xr->y_res = 800 ;
+      xr->y_res = 768 ;
       return XNEE_OK;
     }
   else if (xnee_check ("XGA", res_str, res_str))
@@ -167,7 +150,6 @@ xnee_resolution_differs (xnee_data *xd)
       /* no diff */
       return 0;
     }
-  return XNEE_RESOLUTION_UNSET;
 }
 
 
@@ -175,6 +157,7 @@ int
 xnee_resolution_newx (xnee_data *xd, int xval)
 {
   static int diff = XNEE_RESOLUTION_UNSET ; 
+
   if (diff==XNEE_RESOLUTION_UNSET)  
     {
       if (xnee_is_resolution_used(xd))
@@ -206,6 +189,7 @@ int
 xnee_resolution_newy (xnee_data *xd, int yval)
 {
   static int diff = XNEE_RESOLUTION_UNSET ;
+
   if (diff==XNEE_RESOLUTION_UNSET)
     {
       if (xnee_is_resolution_used(xd))
@@ -235,12 +219,20 @@ xnee_resolution_newy (xnee_data *xd, int yval)
 int
 xnee_is_resolution_used (xnee_data *xd)
 {
+   if (xd==NULL)
+   {
+      return XNEE_MEMORY_FAULT;
+   }
   return xd->res_info.is_used==XNEE_RESOLUTION_USED;  
 }
 
 int 
 xnee_set_resolution_used (xnee_data *xd)
 {
+   if (xd==NULL)
+   {
+      return XNEE_MEMORY_FAULT;
+   }
   xd->res_info.is_used=XNEE_RESOLUTION_USED;  
   return XNEE_OK;
 }
@@ -248,6 +240,10 @@ xnee_set_resolution_used (xnee_data *xd)
 int 
 xnee_unset_resolution_used (xnee_data *xd)
 {
+   if (xd==NULL)
+   {
+      return XNEE_MEMORY_FAULT;
+   }
   xd->res_info.is_used=XNEE_RESOLUTION_UNUSED;  
   return XNEE_OK;
 }
@@ -255,17 +251,23 @@ xnee_unset_resolution_used (xnee_data *xd)
 int 
 xnee_get_resolution_used (xnee_data *xd)
 {
+   if (xd==NULL)
+   {
+      return XNEE_MEMORY_FAULT;
+   }
   return xd->res_info.is_used;  
 }
 
 int
 xnee_resolution_init (xnee_data *xd)
 {
-  xd->res_info.record.x_res=0;
-  xd->res_info.record.y_res=0;
-  xd->res_info.replay.x_res=0;
-  xd->res_info.replay.y_res=0;
-  xnee_set_resolution_used(xd);
+   xd->res_info.record.x_res=1; 
+   xd->res_info.record.y_res=1; 
+
+   xd->res_info.replay.x_res=1; 
+   xd->res_info.replay.y_res=1; 
+
+/*   xnee_set_resolution_used(xd); */
   return XNEE_OK;
 }
  
